@@ -5,12 +5,14 @@ module Schleuder
     include Singleton
 
     EMAIL_REGEXP = /\A.+@[[:alnum:]_.-]+\z/i
-    FINGERPRINT_REGEXP = /\A(0x)?[a-f0-9]{32,}\z/i
+    # TODO: drop v3 keys and only accept length of 40
+    FINGERPRINT_REGEXP = /\A(0x)?[a-f0-9]{32}([a-f0-9]{8})?\z/i
 
     DEFAULTS = {
       'lists_dir' => '/var/lib/schleuder/lists',
       'listlogs_dir' => '/var/lib/schleuder/lists',
       'plugins_dir' => '/etc/schleuder/plugins',
+      'filters_dir' => '/usr/local/lib/schleuder/filters',
       'log_level' => 'warn',
       'superadmin' => 'root@localhost',
       'keyserver' => 'hkp://pool.sks-keyservers.net',
@@ -58,6 +60,10 @@ module Schleuder
       instance.config['plugins_dir']
     end
 
+    def self.filters_dir
+      instance.config['filters_dir']
+    end
+
     def self.database
       instance.config['database'][ENV['SCHLEUDER_ENV']]
     end
