@@ -124,7 +124,7 @@ module Schleuder
 
     def import_key_and_find_fingerprint(key_material)
       return nil if key_material.blank?
-      
+
       import_result = import_key(key_material)
       gpg.interpret_import_result(import_result)
     end
@@ -251,9 +251,8 @@ module Schleuder
     end
 
     def fingerprint=(arg)
-      # Strip whitespace from incoming arg.
       if arg
-        write_attribute(:fingerprint, arg.gsub(/\s*/, '').chomp)
+        write_attribute(:fingerprint, arg.gsub(/\s*/, '').gsub(/^0x/, '').chomp.upcase)
       end
     end
 
@@ -346,7 +345,11 @@ module Schleuder
       mail.add_internal_footer!
       self.subscriptions.each do |subscription|
         begin
-          subscription.send_mail(mail)
+          if subscription.delivery_enabled
+            subscription.send_mail(mail)
+          else
+            logger.info "Not sending to #{subscription.email}: delivery is disabled."
+          end
         rescue => exc
           msg = I18n.t('errors.delivery_error',
                        { email: subscription.email, error: exc.to_s })
